// recv511.cpp //

// Change History ------------------------------------------------------------
//
//	version  date     by  description
//
//	00.00.03 10/11/94 jpc cleaned up for distribution to quics
//	00.00.02 01/07/94 jpc added error reporting statistics
//	00.00.01 12/15/93 jpc re-created
//	00.00.00 ??/??/93 brc created module
//
// End Change History --------------------------------------------------------


// Contents ------------------------------------------------------------------
//
// Description
//
//		This process implements the receiver to be used as part of a
//		transmission media test suite. It will accept a stream of bytes
//		that it expects to be the classic 511-test pattern. This pattern
//		has two advantages:
//
//			1) Given a seed value, the next byte is predictable
//			2) It will generate a wide range of byte patterns.
//
// End -----------------------------------------------------------------------


#include "iostream.h"
#include "iomanip.h"
#include "stdio.h"
#include "sys/types.h"
#include "unistd.h"


// Usage Message //

#ifdef __USAGE
%C -  511-test pattern receiver, error reporter.

%C   <  [source]
Options:
source      Where to read 511-test bytes from

Version:    00.00.03
#endif


// Contants //

#define	RD_SIZE		10240	// max read chunk


// Prototypes //

uchar_t get511( uchar_t b );



// Function //

void main( void )

// Summary -------------------------------------------------------------------
//
//		This function drives the receiver. If an error is encountered,
//		this function will output a report of the form:
//
//			Rxd: 000, Errs: 000, Last: 000, 00.0%
//
//		Where:
//			Rxd		total number of bytes received
//			Errs	total number of errors
//			Last	bytes received since last error
//			00.0%	errors as a percentage of bytes received
//
// End -----------------------------------------------------------------------
	{
	ulong_t rx_cnt   = 0;	// number of bytes read
	ulong_t rx_err   = 0;	// total number of errors encountered
	ulong_t last_err = 0;	// byte count of last receive error


	uchar_t next;			// next byte we expect to see
	read( 0, &next, 1 );	// seed 'next' with a byte
	next = get511( next );	// anticipate next byte pattern

	while(1)				// forever
		{
		uchar_t buff[ RD_SIZE ];
		int count = read( 0, buff, RD_SIZE );

		if ( count > 0 )
			{
			uchar_t* bptr = buff;
			uchar_t* eptr = bptr + count;

			while( bptr < eptr )
				{
				// If next byte pattern was not what we expected, 
				// generate an error report

				if ( *bptr != next )
					{
					ulong_t bcnt = rx_cnt + (bptr-buff);
					cerr << "Rxd:"    << bcnt;
					cerr << ", Errs:" << ++rx_err;
					cerr << ", Last:" << setw(5) << (bcnt-last_err);
					last_err = bcnt;
					float pct_err = (rx_err / float(bcnt)) * 100.0;
					cerr << ", " << setprecision(5) << pct_err << " %";
					cerr << endl;
					}

				next = get511( *bptr );
				bptr++;
				}
			}
		rx_cnt += count;
		}
	}
// End Function main //


// Function //

uchar_t get511( uchar_t b )

// Summary -------------------------------------------------------------------
//
//		This method implements a variant of the 511 Test Pattern. This is
//		useful for sending a byte where the next byte is algorithmically
//		implied. It can be used to send a stream of bytes over a 
//		communications link. A reciever may then synchronize itself with
//		first byte, and know the subsequent byte sequences. If the next
//		byte is not the expected one, then the media is non-reliable.
//		The bad byte, however, can be used to re-syncronize the data
//		stream.
//
// Parameters
//
//		b		The byte to use for the next transition
//
// Return Value
//
//		The transitioned byte.
//
// Functional Description
//
//		The algorithm used to transform the byte is:
//
//			rotate the byte left
//			XOR the byte with the least signifigant bit shifted left 5.
//
// End -----------------------------------------------------------------------
	{
	if ( b & 0x80 )
		{
		b <<= 1;
		b  |= 1;
		b  ^= 0x20;
		}
	else{
		b <<= 1;
		}
	return b;
	}
// End Function get511 //
