// send511.cpp //

// Change History ------------------------------------------------------------
//
//	version  date     by  description
//
//	00.00.03 10/11/94 jpc cleaned up for distribution to quics
//	00.00.02 01/07/94 jpc added error bytes/sec reporting
//	00.00.01 12/15/93 jpc re-created
//	00.00.00 ??/??/93 brc created module
//
// End Change History --------------------------------------------------------


// Contents ------------------------------------------------------------------
//
// Description
//
//		This process implements the sender to be used as part of a
//		transmission media test suite. It will generate a stream of
//		bytes using the classic 511-test pattern algorithm. This pattern
//		has two advantages:
//
//			1) Given a seed value, the next byte is predictable
//			2) It will generate a wide range of byte patterns.
//
// End -----------------------------------------------------------------------


#include "iomanip.h"
#include "iostream.h"
#include "math.h"
#include "stdio.h"
#include "stdlib.h"
#include "time.h"
#include "unistd.h"


// Usage Message //

#ifdef __USAGE
%C - 511-test pattern generator, bps reporter

%C [write]  > [output]
Options:
write       Number of 511-test bytes to generate in one write
output      Where to send 511-test bytes

Version:    00.00.03
#endif


// Types //

// The class that follows is used to keep track of a running standard
// deviation. It is included in this module to allow us to track the
// mean. Note: At PETS, this class has it's own seperate header file,
// but we have included it here since it is manageable enough to fit
// in a small source file.
//
// For a description of how to compute standard deviation "on-the-fly"
// see any good book on basic statistics.

class StdDev
{
private:
		ulong_t	sum;		// sum of the samples
		ulong_t	sq;			// sum of the squared samples
		short	count;		// number of samples

public:
				StdDev()			{ sum=sq=0; count=0; }
		void	Reset()				{ sum=sq=0; count=0; }
		void	Sample( ulong_t n )	{ sum+=n; sq+=(n*n); count++; }
		ulong_t	Deviation() const	// compute std dev
					{
					return (count==0) ? 0 :
					sqrt( (sq-((sum*sum)/count))/count );
					}
		ulong_t	Mean() const		// compute average
					{
					return (count==0) ? 0 : sum/count;
					}
};


// Contants //

#define WR_SIZE		1024	// default write chunk size (in bytes)


// Prototypes //

uchar_t get511( uchar_t b );


// Function //

void main( int argc, const char* const argv[] )

// Summary -------------------------------------------------------------------
//
//		The main entry point from the OS
//
// End -----------------------------------------------------------------------
	{
	int wr_size = WR_SIZE;		// size of write buffer
	if ( argc > 1 )
		{
		wr_size = atoi(argv[1]);
		if ( wr_size < 1 )
			{
			cerr<<"invalid write size specified\a"<<endl;
			return;
			}
		}

	uchar_t* buff = new uchar_t[ wr_size ];
	if ( !buff )
		{
		cerr<<"unable to allocate output write buffer\a"<<endl;
		return;
		}
	cerr<<"using buffer size of "<<wr_size<<endl;

	
	StdDev tx;			// average transmit time
	char wheel = '*';	// activity indicator
	uchar_t b  = 0x40;	// send seed value

	while(1)			// forever
		{
		// generate a buffer full of byte patterns
		uchar_t* bptr = buff;
		uchar_t* eptr = bptr + wr_size;

		while( bptr < eptr )
			{
			*bptr++ = b;
			b = get511( b );
			}

		// write out the byte pattern buffer, and time the length of
		// the write.

		clock_t st = clock();
		if ( write( 1, buff, wr_size ) != wr_size )
			{
			cerr<<"write error encountered\a"<<endl;
			return;
			}
		clock_t et = clock();

		// Toggle the 'wheel' value back and forth between 'o' and '*'. 
		// This will allow us to output a sort-of activity indicator, to 
		// know that this process is working.
	
		cerr<<'\r'<<wheel;
		wheel ^= 0x45;		// XOR with   0100 0101

		// We use the StdDev object to keep a running average of the
		// transmission speed. Each new sample (length of write time in
		// clock ticks), is added to the population. These samples
		// are averaged, then converted into bits/sec before being
		// displayed.

		clock_t diff = et - st;
		tx.Sample( diff );
		double avg = wr_size / (tx.Mean() / double(CLOCKS_PER_SEC));
		cerr<<setprecision(2)<<setw(10)<<avg<<" bps";
		cerr.flush();
		}
	}
// End Function main //


// Function //

uchar_t get511( uchar_t b )

// Summary -------------------------------------------------------------------
//
//		This method implements a variant of the 511 Test Pattern. This is
//		useful for sending a byte where the next byte is algorithmically
//		implied. It can be used to send a stream of bytes over a 
//		communications link. A reciever may then synchronize itself with
//		first byte, and know the subsequent byte sequences. If the next
//		byte is not the expected one, then the media is non-reliable.
//		The bad byte, however, can be used to re-syncronize the data
//		stream.
//
// Parameters
//
//		b		The byte to use for the next transition
//
// Return Value
//
//		The transitioned byte.
//
// Functional Description
//
//		The algorithm used to transform the byte is:
//
//			rotate the byte left
//			XOR the byte with the least signifigant bit shifted left 5.
//
// End -----------------------------------------------------------------------
	{
	if ( b & 0x80 )
		{
		b <<= 1;
		b  |= 1;
		b  ^= 0x20;
		}
	else{
		b <<= 1;
		}
	return b;
	}
// End Function get511 //
